/*
    Copyright (C) 2005-2012  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include "gui.h"
#include "nntpgrab_utils.h"

#ifdef HAVE_SOUP

enum {
    FIELD_COLLECTION_ID,
    FIELD_COLLECTION_NAME,
    FIELD_SHORT_COLLECTION_NAME,
    FIELD_SUBJECT,
    FIELD_POSTER,
    FIELD_NEWSGROUP,
    FIELD_FILE_ID,
    FIELD_FILESIZE_STR,
    FIELD_STAMP,
    FIELD_STAMP_STR,
    FIELD_COMPLETE_PERCENTAGE,
    FIELD_SELECTED,
    LAST_FIELD
};
static GtkTreeViewColumn *columns[LAST_FIELD] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
static GThreadPool *pool_details = NULL;
static NZBCreatorSearchOpts cached_search_opts;
static int results_per_page = 0;

// nzb.c
gboolean    nzb_perform_quick_import(const char *collection_name, const char *nzb_contents);

static gboolean
process_error(gpointer data)
{
    GtkWidget *btnNZBCreatorPerformSearch = nntpgrab_gui_base_get_widget("btnNZBCreatorPerformSearch");
    GtkWidget *windowMain = nntpgrab_gui_base_get_widget("windowMain");
    GtkWidget *entryNZBCreatorQuery = nntpgrab_gui_base_get_widget("entryNZBCreatorQuery");
    GtkWidget *expanderNZBCreator = nntpgrab_gui_base_get_widget("expanderNZBCreator");
    GtkWidget *treeNZBCreatorResults = nntpgrab_gui_base_get_widget("treeNZBCreatorResults");
    GtkWidget *btnNZBCreatorNewerResults = nntpgrab_gui_base_get_widget("btnNZBCreatorNewerResults");
    GtkWidget *btnNZBCreatorOlderResults = nntpgrab_gui_base_get_widget("btnNZBCreatorOlderResults");
    GtkWidget *btnNZBCreatorSelectAll = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectAll");
    GtkWidget *btnNZBCreatorSelectNone = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectNone");
    char *msg = (char*) data;

    nntpgrab_gui_base_dialog_show(windowMain, msg, GTK_MESSAGE_INFO, GTK_BUTTONS_OK);

    g_free(msg);

    gtk_widget_set_sensitive(btnNZBCreatorPerformSearch, FALSE);
    gtk_widget_set_sensitive(entryNZBCreatorQuery, FALSE);
    gtk_widget_set_sensitive(expanderNZBCreator, FALSE);
    gtk_widget_set_sensitive(treeNZBCreatorResults, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorNewerResults, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorOlderResults, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorSelectAll, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorSelectNone, FALSE);

    return FALSE;
}

static void
generate_stamp(time_t stamp, char *buf, int buf_len)
{
    struct tm *now;

    if ((now = localtime(&stamp)) == NULL) {
        // Date could not be parted
        buf[0] = '\0';
    }

    strftime(buf, buf_len, "%c", now);
}

static gboolean
process_search_result(gpointer data)
{
    GtkWidget *btnNZBCreatorPerformSearch = nntpgrab_gui_base_get_widget("btnNZBCreatorPerformSearch");
    GtkWidget *entryNZBCreatorQuery = nntpgrab_gui_base_get_widget("entryNZBCreatorQuery");
    GtkWidget *expanderNZBCreator = nntpgrab_gui_base_get_widget("expanderNZBCreator");
    GtkWidget *treeNZBCreatorResults = nntpgrab_gui_base_get_widget("treeNZBCreatorResults");
    GtkWidget *btnNZBCreatorNewerResults = nntpgrab_gui_base_get_widget("btnNZBCreatorNewerResults");
    GtkWidget *btnNZBCreatorOlderResults = nntpgrab_gui_base_get_widget("btnNZBCreatorOlderResults");
    GtkWidget *btnNZBCreatorSelectAll = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectAll");
    GtkWidget *btnNZBCreatorSelectNone = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectNone");
    NZBCreatorSearchResult *result = (NZBCreatorSearchResult*) data;

    if (result->number_of_results == 0) {
        GtkWidget *windowMain = nntpgrab_gui_base_get_widget("windowMain");
        nntpgrab_gui_base_dialog_show(windowMain, _("No results found"), GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
    } else {
        NGList *list;
        GtkTreeModel *model;
        GtkTreeIter iter_parent;

        model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeNZBCreatorResults));

        list = result->collections;
        while (list) {
            char filesize[64];
            char stamp[64];
            NZBCreatorCollection *collection = (NZBCreatorCollection*) list->data;

            memset(&filesize, 0, sizeof(filesize));
            nntpgrab_utils_calculate_file_size(collection->total_size, filesize, sizeof(filesize) - 1);

            memset(&stamp, 0, sizeof(stamp));
            generate_stamp(collection->stamp, stamp, sizeof(stamp) - 1);

            gtk_tree_store_append(GTK_TREE_STORE(model), &iter_parent, NULL);
            gtk_tree_store_set(GTK_TREE_STORE(model), &iter_parent,
                                    FIELD_COLLECTION_ID, collection->collection_id,
                                    FIELD_COLLECTION_NAME, collection->collection_name,
                                    FIELD_SHORT_COLLECTION_NAME, collection->short_collection_name,
                                    FIELD_SUBJECT, collection->collection_name,
                                    FIELD_POSTER, collection->poster,
                                    FIELD_NEWSGROUP, collection->newsgroup,
                                    FIELD_STAMP, (int) collection->stamp,
                                    FIELD_STAMP_STR, stamp,
                                    FIELD_FILESIZE_STR, filesize,
                                    FIELD_FILE_ID, collection->first_file_id,
                                    -1);

            if (collection->first_file_id == -1) {
                /* Show a placeholder item */
                GtkTreeIter iter;

                gtk_tree_store_append(GTK_TREE_STORE(model), &iter, &iter_parent);
                gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                                   FIELD_SUBJECT, "unknown",
                                   -1);
            }

            gtk_tree_store_set(GTK_TREE_STORE(model), &iter_parent, FIELD_COMPLETE_PERCENTAGE, collection->complete_percentage, -1);

            list = ng_list_next(list);
        }

        gtk_expander_set_expanded(GTK_EXPANDER(expanderNZBCreator), FALSE);

        if (result->number_of_results > result->results_per_page && result->offset + result->results_per_page < result->number_of_results) {
            gtk_widget_set_sensitive(btnNZBCreatorOlderResults, TRUE);
        }

        gtk_widget_set_sensitive(btnNZBCreatorSelectAll, TRUE);
        gtk_widget_set_sensitive(btnNZBCreatorSelectNone, TRUE);
    }

    results_per_page = result->results_per_page;

    if (result->offset > 0) {
        gtk_widget_set_sensitive(btnNZBCreatorNewerResults, TRUE);
    }

    gtk_widget_set_sensitive(btnNZBCreatorPerformSearch, TRUE);
    gtk_widget_set_sensitive(entryNZBCreatorQuery, TRUE);
    gtk_widget_set_sensitive(expanderNZBCreator, TRUE);
    gtk_widget_set_sensitive(treeNZBCreatorResults, TRUE);
    return FALSE;
}

static gpointer
perform_search_func(gpointer data)
{
    NZBCreatorSearchOpts *opts = (NZBCreatorSearchOpts*) data;
    NZBCreatorSearchResult *result;
    char *errmsg = NULL;

    result = nntpgrab_utils_nzbcreator_perform_search(*opts, &errmsg);
    if (result) {
        g_idle_add(process_search_result, result);
    } else {
        char *msg = g_strdup_printf(_("Error while performing search:\n%s"), errmsg);
        g_idle_add(process_error, msg);
        g_free(errmsg);
    }

    return NULL;
}

static void
perform_search(void)
{
    GtkWidget *entryNZBCreatorQuery = nntpgrab_gui_base_get_widget("entryNZBCreatorQuery");
    GtkWidget *treeNZBCreatorResults = nntpgrab_gui_base_get_widget("treeNZBCreatorResults");
    GtkWidget *expanderNZBCreator = nntpgrab_gui_base_get_widget("expanderNZBCreator");
    GtkWidget *btnNZBCreatorPerformSearch = nntpgrab_gui_base_get_widget("btnNZBCreatorPerformSearch");
    GtkWidget *btnNZBCreatorImportFiles = nntpgrab_gui_base_get_widget("btnNZBCreatorImportFiles");
    GtkWidget *btnNZBCreatorNewerResults = nntpgrab_gui_base_get_widget("btnNZBCreatorNewerResults");
    GtkWidget *btnNZBCreatorOlderResults = nntpgrab_gui_base_get_widget("btnNZBCreatorOlderResults");
    GtkWidget *btnNZBCreatorSelectAll = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectAll");
    GtkWidget *btnNZBCreatorSelectNone = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectNone");
    GtkTreeModel *model;

    gtk_widget_set_sensitive(expanderNZBCreator, FALSE);
    gtk_widget_set_sensitive(entryNZBCreatorQuery, FALSE);
    gtk_widget_set_sensitive(treeNZBCreatorResults, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorPerformSearch, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorImportFiles, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorNewerResults, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorOlderResults, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorSelectAll, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorSelectNone, FALSE);

    /* Clear any old results */
    model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeNZBCreatorResults));
    gtk_tree_store_clear(GTK_TREE_STORE(model));

    g_return_if_fail(g_thread_create(perform_search_func, &cached_search_opts, FALSE, NULL));
}

struct _nzbcreator_data
{
    time_t oldest_stamp;
    time_t newest_stamp;
    NGList *file_ids;
    char *collection_name;
    char *nzbdata;
    char *errmsg;
};

static gboolean
process_nzbcreator_results(gpointer user_data)
{
    struct _nzbcreator_data *data = (struct _nzbcreator_data*) user_data;
    GtkWidget *treeNZBCreatorResults = nntpgrab_gui_base_get_widget("treeNZBCreatorResults");
    GtkWidget *btnNZBCreatorPerformSearch = nntpgrab_gui_base_get_widget("btnNZBCreatorPerformSearch");
    GtkWidget *btnNZBCreatorImportFiles = nntpgrab_gui_base_get_widget("btnNZBCreatorImportFiles");
    GtkWidget *entryNZBCreatorQuery = nntpgrab_gui_base_get_widget("entryNZBCreatorQuery");
    GtkWidget *expanderNZBCreator = nntpgrab_gui_base_get_widget("expanderNZBCreator");
    GtkWidget *btnNZBCreatorNewerResults = nntpgrab_gui_base_get_widget("btnNZBCreatorNewerResults");
    GtkWidget *btnNZBCreatorOlderResults = nntpgrab_gui_base_get_widget("btnNZBCreatorOlderResults");
    GtkWidget *btnNZBCreatorSelectAll = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectAll");
    GtkWidget *btnNZBCreatorSelectNone = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectNone");

    if (!data->nzbdata) {
        GtkWidget *windowMain = nntpgrab_gui_base_get_widget("windowMain");
        nntpgrab_gui_base_dialog_show(windowMain, data->errmsg, GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
        g_free(data->errmsg);

        goto out;
    }

    nzb_perform_quick_import(data->collection_name, data->nzbdata);

out:
    gtk_widget_set_sensitive(treeNZBCreatorResults, TRUE);
    gtk_widget_set_sensitive(btnNZBCreatorPerformSearch, TRUE);
    gtk_widget_set_sensitive(btnNZBCreatorImportFiles, TRUE);
    gtk_widget_set_sensitive(entryNZBCreatorQuery, TRUE);
    gtk_widget_set_sensitive(expanderNZBCreator, TRUE);
    gtk_widget_set_sensitive(btnNZBCreatorNewerResults, TRUE);
    gtk_widget_set_sensitive(btnNZBCreatorOlderResults, TRUE);
    gtk_widget_set_sensitive(btnNZBCreatorSelectAll, TRUE);
    gtk_widget_set_sensitive(btnNZBCreatorSelectNone, TRUE);

    g_free(data->nzbdata);
    g_free(data->collection_name);
    ng_list_free(data->file_ids);
    g_slice_free(struct _nzbcreator_data, data);

    return FALSE;
}

static gpointer
retrieve_nzbcreator_results(gpointer user_data)
{
    struct _nzbcreator_data *data = (struct _nzbcreator_data*) user_data;

    data->nzbdata = nntpgrab_utils_nzbcreator_generate_NZB(data->file_ids, data->oldest_stamp, data->newest_stamp, &data->errmsg);

    g_idle_add(process_nzbcreator_results, data);

    return NULL;
}

static gboolean
process_newsgroups_list(gpointer data)
{
    NGList *newsgroups = (NGList*) data;
    NGList *list;
    GtkWidget *comboNZBCreatorNewsgroups;
    GtkWidget *vboxNZBCreatorTab;
    GtkListStore *store;
    GtkTreeIter iter;
    GtkCellRenderer *renderer;

    comboNZBCreatorNewsgroups = nntpgrab_gui_base_get_widget("comboNZBCreatorNewsgroups");
    vboxNZBCreatorTab = nntpgrab_gui_base_get_widget("vboxNZBCreatorTab");

    store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);

    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, _("All popular newsgroups"), -1);

    list = newsgroups;
    while (list) {
        NZBCreatorGroup *group = (NZBCreatorGroup*) list->data;

        gtk_list_store_append(store, &iter);
        gtk_list_store_set(store, &iter, 0, group->newsgroup, 1, group->group_id, -1);

        list = ng_list_next(list);
    }

    gtk_combo_box_set_model(GTK_COMBO_BOX(comboNZBCreatorNewsgroups), GTK_TREE_MODEL(store));
    gtk_combo_box_set_active(GTK_COMBO_BOX(comboNZBCreatorNewsgroups), 0);

    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (comboNZBCreatorNewsgroups), renderer, TRUE);
    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (comboNZBCreatorNewsgroups), renderer, "text", 0, NULL);

    gtk_widget_set_sensitive(vboxNZBCreatorTab, TRUE);

    nntpgrab_utils_nzbcreator_free_groups(newsgroups);

    return FALSE;
}

static gboolean
disable_online_search(gpointer data)
{
    GtkTreeIter iter;
    GtkWidget *treeNZBCreatorResults = nntpgrab_gui_base_get_widget("treeNZBCreatorResults");
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeNZBCreatorResults));

    g_return_val_if_fail(data != NULL, FALSE);

    gtk_tree_store_append(GTK_TREE_STORE(model), &iter, NULL);
    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                            FIELD_SUBJECT, data,
                            -1);

    g_free(data);

    return FALSE;
}

static gpointer
fetch_newsgroups_func(gpointer data)
{
    NGList *newsgroups;
    gboolean api_okay = FALSE;
    NZBCreatorMaintenanceInfo *info;
    char *errmsg = NULL;

    if (!nntpgrab_utils_nzbcreator_check_api_version(&api_okay, &errmsg)) {
        /* Some error occured while checking the API version */
        g_idle_add(disable_online_search, g_strdup_printf(_("Unexpected error occured while connecting to the Online Search:\n%s"), errmsg));
        g_free(errmsg);
        return NULL;
    }

    if (!api_okay) {
        /* A newer version of NNTPGrab is required */
        g_idle_add(disable_online_search, g_strdup(_("A newer version of NNTPGrab is required to use the Online Search")));
        return NULL;
    }

    if (!(info = nntpgrab_utils_nzbcreator_get_maintenance_info(&errmsg))) {
        /* Some error occured while retrieving the maintenance information */
        g_idle_add(disable_online_search, g_strdup_printf(_("Unexpected error occured while retrieving maintenance information the Online Search:\n%s"), errmsg));
        g_free(errmsg);
        return NULL;
    }

    if (info->down_for_maintenance) {
        g_idle_add(disable_online_search, g_strdup(info->reason));
        nntpgrab_utils_nzbcreator_free_maintenance_info(info);
        return NULL;
    }
    nntpgrab_utils_nzbcreator_free_maintenance_info(info);

    newsgroups = nntpgrab_utils_nzbcreator_get_all_groups(&errmsg);
    if (newsgroups) {
        g_idle_add(process_newsgroups_list, newsgroups);
    } else {
        char *msg = g_strdup_printf(_("Error while fetching list of newsgroups from the Online Search service:\n%s"), errmsg);
        g_idle_add(process_error, msg);
        g_free(errmsg);
    }

    return NULL;
}

struct _details_data {
    GtkTreeStore *store;
    GtkTreeIter iter_parent;
    ngint64 collection_id;
    time_t stamp;

    char *errmsg;

    NGList *result;
};

static void
on_row_expanded(GtkTreeView *tree_view, GtkTreeIter *iter_parent, GtkTreePath *path, gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
    struct _details_data *details_data = g_slice_new0(struct _details_data);
    char *subject = NULL;

    if (!gtk_tree_model_iter_children(model, &iter, iter_parent)) {
        /* This is a single-file collection, no need to do anything with it */
        return;
    }

    gtk_tree_model_get(model, iter_parent, FIELD_COLLECTION_ID, &details_data->collection_id, FIELD_STAMP, &details_data->stamp, -1);
    gtk_tree_model_get(model, &iter, FIELD_SUBJECT, &subject, -1);

    if (subject && !strcmp(subject, "unknown")) {
        /* No information retrieved yet for this collection, retrieve it right now */
        gtk_tree_store_set(GTK_TREE_STORE(model), &iter, FIELD_SUBJECT, _("No information available yet. Please wait a moment..."), -1);

        g_return_if_fail(pool_details != NULL);

        details_data->store = GTK_TREE_STORE(model);
        details_data->iter_parent = *iter_parent;
        g_thread_pool_push(pool_details, details_data, NULL);
    }
}

static void
on_column_select_toggled(GtkWidget *caller, char *path, gpointer data)
{
    GtkTreeModel *model = GTK_TREE_MODEL(data);
    GtkTreeIter iter;
    GtkTreeIter parent;
    gboolean val;
    GtkWidget *btnNZBCreatorImportFiles;
    gboolean selected;

    gtk_tree_model_get_iter_from_string(model, &iter, path);

    gtk_tree_model_get(model, &iter, FIELD_SELECTED, &val, -1);
    if (val) {
        val = FALSE;
    } else {
        val = TRUE;
    }

    // Has the user clicked on the checkbox of a collection or just a single file?
    if (gtk_tree_model_iter_parent(model, &parent, &iter)) {
        // Single file
        gtk_tree_store_set(GTK_TREE_STORE(model), &iter, FIELD_SELECTED, val, -1);

        /* Some collections only contain one file (thus no expander). Such rows can be ignored */
        if (gtk_tree_model_iter_children(model, &iter, &parent)) {
            do {
                gtk_tree_model_get(model, &iter, FIELD_SELECTED, &selected, -1);
                if(selected) {
                    //There are items selected
                    break;
                }
            } while (gtk_tree_model_iter_next(model, &iter));
            if(!selected) {
                //No items selected in this collection
                gtk_tree_store_set(GTK_TREE_STORE(model), &parent, FIELD_SELECTED, val, -1);
            }
        }
    } else {
        // Whole collection
        parent = iter;

        gtk_tree_store_set(GTK_TREE_STORE(model), &parent, FIELD_SELECTED, val, -1);

        /* Some collections only contain one file (thus no expander). Such rows can be ignored */
        if (gtk_tree_model_iter_children(model, &iter, &parent)) {
            do {
                gtk_tree_store_set(GTK_TREE_STORE(model), &iter, FIELD_SELECTED, val, -1);
            } while (gtk_tree_model_iter_next(model, &iter));
        }

        /* Retrieve the collection details if the collection wasn't expanded earlier yet */
        if (val) {
            on_row_expanded(GTK_TREE_VIEW(nntpgrab_gui_base_get_widget("treeNZBCreatorResults")), &parent, NULL, NULL);
        }
    }

    // Is at least one item selected?
    g_return_if_fail(gtk_tree_model_get_iter_first(model, &parent));

    do {
        gtk_tree_model_get(model, &parent, FIELD_SELECTED, &val, -1);
        if (val) {
            btnNZBCreatorImportFiles = nntpgrab_gui_base_get_widget("btnNZBCreatorImportFiles");
            gtk_widget_set_sensitive(btnNZBCreatorImportFiles, TRUE);
            return;
        }

        if (gtk_tree_model_iter_children(model, &iter, &parent)) {
            do {
                gtk_tree_model_get(model, &iter, FIELD_SELECTED, &val, -1);
                if (val) {
                    btnNZBCreatorImportFiles = nntpgrab_gui_base_get_widget("btnNZBCreatorImportFiles");
                    gtk_widget_set_sensitive(btnNZBCreatorImportFiles, TRUE);
                    return;
                }
            } while (gtk_tree_model_iter_next(model, &iter));
        }
    } while (gtk_tree_model_iter_next(model, &parent));

    btnNZBCreatorImportFiles = nntpgrab_gui_base_get_widget("btnNZBCreatorImportFiles");
    gtk_widget_set_sensitive(btnNZBCreatorImportFiles, FALSE);
}

static gboolean
process_collection_details(gpointer data)
{
    NGList *list;
    char filesize[64];
    char stamp[64];
    char *newsgroup;
    GtkTreeIter iter;
    gboolean selected = FALSE;
    struct _details_data *details_data = (struct _details_data *) data;

    g_return_val_if_fail(details_data != NULL, FALSE);

    if (details_data->errmsg != NULL) {
        GtkWidget *windowMain = nntpgrab_gui_base_get_widget("windowMain");

        nntpgrab_gui_base_dialog_show(windowMain, details_data->errmsg, GTK_MESSAGE_INFO, GTK_BUTTONS_OK);

        g_free(details_data->errmsg);
        goto out;
    }

    gtk_tree_model_get(GTK_TREE_MODEL(details_data->store), &details_data->iter_parent, FIELD_NEWSGROUP, &newsgroup, FIELD_SELECTED, &selected, -1);

    list = details_data->result;
    while (list) {
        NZBCreatorFile *file = (NZBCreatorFile*) list->data;

        memset(&filesize, 0, sizeof(filesize));
        nntpgrab_utils_calculate_file_size(file->file_size, filesize, sizeof(filesize) - 1);

        memset(&stamp, 0, sizeof(stamp));
        generate_stamp(file->stamp, stamp, sizeof(stamp) - 1);

        gtk_tree_store_append(details_data->store, &iter, &details_data->iter_parent);
        gtk_tree_store_set(details_data->store, &iter,
                                FIELD_SUBJECT, file->subject,
                                FIELD_POSTER, file->poster,
                                FIELD_NEWSGROUP, newsgroup,
                                FIELD_FILE_ID, file->file_id,
                                FIELD_FILESIZE_STR, filesize,
                                FIELD_STAMP, file->stamp,
                                FIELD_STAMP_STR, stamp,
                                FIELD_COMPLETE_PERCENTAGE, file->complete_percentage,
                                FIELD_SELECTED, selected,
                                -1);

        list = ng_list_next(list);
    }

    /* Remove the placeholder */
    g_return_val_if_fail(gtk_tree_model_iter_children(GTK_TREE_MODEL(details_data->store), &iter, &details_data->iter_parent), FALSE);
    gtk_tree_store_remove(details_data->store, &iter);

    g_free(newsgroup);

out:
    g_slice_free(struct _details_data, details_data);

    return FALSE;
}

static void
get_collection_details_thread(gpointer data, gpointer user_data)
{
    struct _details_data *details_data = (struct _details_data *) data;

    g_return_if_fail(details_data != NULL);

    details_data->result = nntpgrab_utils_nzbcreator_retrieve_collection_details(details_data->collection_id, details_data->stamp, &details_data->errmsg);

    /* Process the results back in the main thread */
    g_idle_add(process_collection_details, details_data);
}

static void
select_files_func(gboolean select_all)
{
    GtkWidget *btnNZBCreatorImportFiles = nntpgrab_gui_base_get_widget("btnNZBCreatorImportFiles");
    GtkWidget *treeNZBCreatorResults = nntpgrab_gui_base_get_widget("treeNZBCreatorResults");
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeNZBCreatorResults));
    GtkTreeIter iter;
    GtkTreeIter parent;
    gboolean has_children;

    g_return_if_fail(gtk_tree_model_get_iter_first(model, &parent));

    do {
        gtk_tree_store_set(GTK_TREE_STORE(model), &parent, FIELD_SELECTED, select_all, -1);

        /* Some items don't have an expander */
        if (!(gtk_tree_model_iter_children(model, &iter, &parent))) {
            has_children = FALSE;
            iter = parent;
        } else {
            has_children = TRUE;
        }

        do {
            /* Retrieve collection details if necessary */
            if (select_all) {
                char *subject = NULL;
                gtk_tree_model_get(model, &iter, FIELD_SUBJECT, &subject, -1);

                if (subject && !strcmp(subject, "unknown")) {
                    /* This is a placeholder, automatically try to retrieve additional details */
                    on_row_expanded(GTK_TREE_VIEW(treeNZBCreatorResults), &parent, NULL, NULL);
                }

                g_free(subject);
            }

            gtk_tree_store_set(GTK_TREE_STORE(model), &iter, FIELD_SELECTED, select_all, -1);

            if (!has_children) {
                break;
            }
        } while (gtk_tree_model_iter_next(model, &iter));
    } while (gtk_tree_model_iter_next(model, &parent));

    gtk_widget_set_sensitive(btnNZBCreatorImportFiles, select_all);
}

#endif

G_MODULE_EXPORT void
on_btnNZBCreatorPerformSearch_clicked(GtkWidget *caller, gpointer data)
{
#ifdef HAVE_SOUP
    NZBCreatorSearchOpts opts;
    GtkWidget *entryNZBCreatorQuery = nntpgrab_gui_base_get_widget("entryNZBCreatorQuery");
    GtkWidget *radNZBCreatorSubjects = nntpgrab_gui_base_get_widget("radNZBCreatorSubjects");
    GtkWidget *radNZBCreatorFiles = nntpgrab_gui_base_get_widget("radNZBCreatorFiles");
    GtkWidget *radNZBCreatorPosters = nntpgrab_gui_base_get_widget("radNZBCreatorPosters");
    GtkWidget *comboNZBCreatorMaxAge = nntpgrab_gui_base_get_widget("comboNZBCreatorMaxAge");
    GtkWidget *spinNZBCreatorMinFileSize = nntpgrab_gui_base_get_widget("spinNZBCreatorMinFileSize");
    GtkWidget *spinNZBCreatorMaxFileSize = nntpgrab_gui_base_get_widget("spinNZBCreatorMaxFileSize");
    GtkWidget *comboNZBCreatorMinFileSize = nntpgrab_gui_base_get_widget("comboNZBCreatorMinFileSize");
    GtkWidget *comboNZBCreatorMaxFileSize = nntpgrab_gui_base_get_widget("comboNZBCreatorMaxFileSize");
    GtkWidget *comboNZBCreatorPercentage = nntpgrab_gui_base_get_widget("comboNZBCreatorPercentage");
    GtkWidget *comboNZBCreatorFileType = nntpgrab_gui_base_get_widget("comboNZBCreatorFileType");
    GtkWidget *comboNZBCreatorNewsgroups = nntpgrab_gui_base_get_widget("comboNZBCreatorNewsgroups");
    const char *query;
    int factor;

    // Query
    query = gtk_entry_get_text(GTK_ENTRY(entryNZBCreatorQuery));
    if (!query || strlen(query) == 0) {
        GtkWidget *windowMain = nntpgrab_gui_base_get_widget("windowMain");
        nntpgrab_gui_base_dialog_show(windowMain, _("Please enter a search query first"), GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
        return;
    }

    if (strlen(query) < 4) {
        GtkWidget *windowMain = nntpgrab_gui_base_get_widget("windowMain");
        nntpgrab_gui_base_dialog_show(windowMain, _("The search query must be at least 4 characters long"), GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
        return;
    }

    memset(&opts, 0, sizeof(NZBCreatorSearchOpts));
    strncpy(opts.query, query, sizeof(opts.query) - 1);

    // Search only in
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radNZBCreatorSubjects))) {
        opts.search_only_in = NZBCREATOR_SEARCH_ONLY_IN_SUBJECTS;
    } else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radNZBCreatorFiles))) {
        opts.search_only_in = NZBCREATOR_SEARCH_ONLY_IN_FILENAMES;
    } else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radNZBCreatorPosters))) {
        opts.search_only_in = NZBCREATOR_SEARCH_ONLY_IN_POSTERS;
    } else {
        g_critical("Unknown search_only_in option\n");
        return;
    }

    // Maximum age
    switch(gtk_combo_box_get_active(GTK_COMBO_BOX(comboNZBCreatorMaxAge))) {
        case 0:                 // 7 days
            opts.max_age = 7;
            break;

        case 1:                 // 14 days
            opts.max_age = 14;
            break;

        case 2:                 // 21 days
            opts.max_age = 21;
            break;

        case 3:                 // 50 days
            opts.max_age = 50;
            break;

        case 4:                 // 75 days
            opts.max_age = 75;
            break;

        case 5:                 // 100 days
            opts.max_age = 100;
            break;

        case 6:                 // 125 days
            opts.max_age = 125;
            break;

        case 7:                 // 150 days
            opts.max_age = 150;
            break;

        default:
            g_critical("Unknown max_age option\n");
            return;
    }

    // Minimal file size
    switch(gtk_combo_box_get_active(GTK_COMBO_BOX(comboNZBCreatorMinFileSize))) {
        case 0:
            factor = 1024;
            break;

        case 1:
            factor = 1024 * 1024;
            break;

        case 2:
            factor = 1024 * 1024 * 1024;
            break;

        default:
            g_critical("Unknown min_file_size option\n");
            return;
    }

    opts.minimal_file_size = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinNZBCreatorMinFileSize));
    if (opts.minimal_file_size > 0) {
        opts.minimal_file_size *= factor;
    }

    // Maximal file size
    switch(gtk_combo_box_get_active(GTK_COMBO_BOX(comboNZBCreatorMaxFileSize))) {
        case 0:
            factor = 1024;
            break;

        case 1:
            factor = 1024 * 1024;
            break;

        case 2:
            factor = 1024 * 1024 * 1024;
            break;

        default:
            g_critical("Unknown max_file_size option\n");
            return;
    }

    opts.maximal_file_size = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinNZBCreatorMaxFileSize));
    if (opts.maximal_file_size > 0) {
        opts.maximal_file_size *= factor;
    }

    // Percentage complete
    switch(gtk_combo_box_get_active(GTK_COMBO_BOX(comboNZBCreatorPercentage))) {
        case 0:                 // Any percentage
            opts.percentage_complete = 0;
            break;

        case 1:                 // Only complete files (100%)
            opts.percentage_complete = 100;
            break;

        case 2:                 // More than 95%
            opts.percentage_complete = 95;
            break;

        case 3:                 // More than 90%
            opts.percentage_complete = 90;
            break;

        case 4:                 // More than 85%
            opts.percentage_complete = 85;
            break;

        case 5:                 // More than 80%
            opts.percentage_complete = 80;
            break;

        case 6:                 // More than 70%
            opts.percentage_complete = 70;
            break;

        case 7:                 // More than 60%
            opts.percentage_complete = 60;
            break;

        case 8:                 // More than 50%
            opts.percentage_complete = 50;
            break;

        default:
            g_critical("Unknown percentage option\n");
            return;
    }

    // File type
    switch(gtk_combo_box_get_active(GTK_COMBO_BOX(comboNZBCreatorFileType))) {
        case 0:                 // All file types
            opts.file_type = NNTP_FILE_TYPE_UNKNOWN;
            break;

        case 1:                 // NZB files
            opts.file_type = NNTP_FILE_TYPE_NZB;
            break;

        case 2:                 // PAR files
            opts.file_type = NNTP_FILE_TYPE_PAR;
            break;

        case 3:                 // PAR2 files
            opts.file_type = NNTP_FILE_TYPE_PAR2;
            break;

        case 4:                 // NFO files
            opts.file_type = NNTP_FILE_TYPE_NFO;
            break;

        case 5:                 // SFV files
            opts.file_type = NNTP_FILE_TYPE_SFV;
            break;

        case 6:                 // RAR files
            opts.file_type = NNTP_FILE_TYPE_RAR;
            break;

        default:
            g_critical("Unknown file_type option\n");
            return;
    }

    // Newsgroup
    if (gtk_combo_box_get_active(GTK_COMBO_BOX(comboNZBCreatorNewsgroups)) > 0) {
        GtkTreeIter iter;
        GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(comboNZBCreatorNewsgroups));

        gtk_combo_box_get_active_iter(GTK_COMBO_BOX(comboNZBCreatorNewsgroups), &iter);
        gtk_tree_model_get(model, &iter, 1, &opts.group_to_search, -1);
    }

    /* Search result offset */
    opts.offset = 0;

    /* Keep the search query cached so we can re-use it later */
    memcpy(&cached_search_opts, &opts, sizeof(NZBCreatorSearchOpts));

    perform_search();
#endif
}

G_MODULE_EXPORT void
on_btnNZBCreatorImportFiles_clicked(GtkWidget *caller, gpointer user_data)
{
#ifdef HAVE_SOUP
    GtkWidget *treeNZBCreatorResults = nntpgrab_gui_base_get_widget("treeNZBCreatorResults");
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeNZBCreatorResults));
    GtkWidget *btnNZBCreatorPerformSearch = nntpgrab_gui_base_get_widget("btnNZBCreatorPerformSearch");
    GtkWidget *btnNZBCreatorImportFiles = nntpgrab_gui_base_get_widget("btnNZBCreatorImportFiles");
    GtkWidget *entryNZBCreatorQuery = nntpgrab_gui_base_get_widget("entryNZBCreatorQuery");
    GtkWidget *expanderNZBCreator = nntpgrab_gui_base_get_widget("expanderNZBCreator");
    GtkWidget *btnNZBCreatorNewerResults = nntpgrab_gui_base_get_widget("btnNZBCreatorNewerResults");
    GtkWidget *btnNZBCreatorOlderResults = nntpgrab_gui_base_get_widget("btnNZBCreatorOlderResults");
    GtkWidget *btnNZBCreatorSelectAll = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectAll");
    GtkWidget *btnNZBCreatorSelectNone = nntpgrab_gui_base_get_widget("btnNZBCreatorSelectNone");
    GtkTreeIter iter;
    GtkTreeIter parent;
    struct _nzbcreator_data *data;
    gboolean val;
    int stamp;
    char *collection_name_current = NULL;
    gboolean has_children;
    const char *query = gtk_entry_get_text(GTK_ENTRY(entryNZBCreatorQuery));

    g_return_if_fail(gtk_tree_model_get_iter_first(model, &parent));

    gtk_widget_set_sensitive(treeNZBCreatorResults, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorPerformSearch, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorImportFiles, FALSE);
    gtk_widget_set_sensitive(entryNZBCreatorQuery, FALSE);
    gtk_widget_set_sensitive(expanderNZBCreator, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorNewerResults, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorOlderResults, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorSelectAll, FALSE);
    gtk_widget_set_sensitive(btnNZBCreatorSelectNone, FALSE);

    data = g_slice_new0(struct _nzbcreator_data);

    do {
        gtk_tree_model_get(model, &parent, FIELD_SHORT_COLLECTION_NAME, &collection_name_current, -1);

        /* Some items don't have an expander */
        if (!(gtk_tree_model_iter_children(model, &iter, &parent))) {
            has_children = FALSE;
            iter = parent;
        } else {
            has_children = TRUE;
        }

        do {
            gint64 file_id = -1;

            gtk_tree_model_get(model, &iter, FIELD_FILE_ID, &file_id, FIELD_SELECTED, &val, FIELD_STAMP, &stamp, -1);
            if (val) {
                // Use the name of the collection
                if (!data->collection_name) {
                    data->collection_name = g_strdup(collection_name_current);
                } else {
                    if (strcmp(data->collection_name, collection_name_current)) {
                        // Or use the search query when there are multiple collections involved
                        if (strcmp(data->collection_name, query)) {
                            g_free(data->collection_name);
                            data->collection_name = g_strdup(query);
                        }
                    }
                }

                data->file_ids = ng_list_append(data->file_ids, GINT_TO_POINTER((int)file_id));

                if (stamp > 0 && (data->oldest_stamp == 0 || stamp < data->oldest_stamp)) {
                    data->oldest_stamp = stamp;
                }

                if (data->newest_stamp == 0 || stamp > data->newest_stamp) {
                    data->newest_stamp = stamp;
                }
            }

            if (!has_children) {
                break;
            }
        } while (gtk_tree_model_iter_next(model, &iter));

        g_free(collection_name_current);
    } while (gtk_tree_model_iter_next(model, &parent));

    g_return_if_fail(data->collection_name != NULL);

    /* Perform the retrieval in a seperate thread to avoid a freeze in the GUI */
    g_return_if_fail(g_thread_create(retrieve_nzbcreator_results, data, FALSE, NULL));
#endif
}

G_MODULE_EXPORT void
on_entryNZBCreatorQuery_activate(GtkWidget *caller, gpointer data)
{
#ifdef HAVE_SOUP
    gtk_button_clicked(GTK_BUTTON(nntpgrab_gui_base_get_widget("btnNZBCreatorPerformSearch")));
#endif
}

G_MODULE_EXPORT void
on_btnNZBCreatorSelectAll_clicked(GtkWidget *caller, gpointer data)
{
#ifdef HAVE_SOUP
    select_files_func(TRUE);
#endif
}

G_MODULE_EXPORT void
on_btnNZBCreatorSelectNone_clicked(GtkWidget *caller, gpointer data)
{
#ifdef HAVE_SOUP
    select_files_func(FALSE);
#endif
}

G_MODULE_EXPORT void
on_btnNZBCreatorOlderResults_clicked(GtkWidget *caller, gpointer data)
{
#ifdef HAVE_SOUP
    g_return_if_fail(results_per_page > 0);
    g_return_if_fail(cached_search_opts.offset >= 0);

    cached_search_opts.offset += results_per_page;

    perform_search();
#endif
}

G_MODULE_EXPORT void
on_btnNZBCreatorNewerResults_clicked(GtkWidget *caller, gpointer data)
{
#ifdef HAVE_SOUP
    g_return_if_fail(results_per_page > 0);
    g_return_if_fail(cached_search_opts.offset >= results_per_page);

    cached_search_opts.offset -= results_per_page;

    perform_search();
#endif
}

void
nzbcreator_initialize(void)
{
#ifdef HAVE_SOUP
    GtkWidget *treeNZBCreatorResults;
    GtkWidget *comboNZBCreatorMinFileSize;
    GtkWidget *comboNZBCreatorMaxFileSize;
    GtkWidget *comboNZBCreatorMaxAge;
    GtkWidget *comboNZBCreatorPercentage;
    GtkWidget *comboNZBCreatorFileType;

    GtkTreeStore *store;
    GtkCellRenderer *cellRenderer;

    treeNZBCreatorResults = nntpgrab_gui_base_get_widget("treeNZBCreatorResults");
    comboNZBCreatorMinFileSize = nntpgrab_gui_base_get_widget("comboNZBCreatorMinFileSize");
    comboNZBCreatorMaxFileSize = nntpgrab_gui_base_get_widget("comboNZBCreatorMaxFileSize");
    comboNZBCreatorMaxAge = nntpgrab_gui_base_get_widget("comboNZBCreatorMaxAge");
    comboNZBCreatorPercentage = nntpgrab_gui_base_get_widget("comboNZBCreatorPercentage");
    comboNZBCreatorFileType = nntpgrab_gui_base_get_widget("comboNZBCreatorFileType");

    gtk_combo_box_set_active(GTK_COMBO_BOX(comboNZBCreatorMinFileSize), 1);
    gtk_combo_box_set_active(GTK_COMBO_BOX(comboNZBCreatorMaxFileSize), 1);
    gtk_combo_box_set_active(GTK_COMBO_BOX(comboNZBCreatorMaxAge), 2);
    gtk_combo_box_set_active(GTK_COMBO_BOX(comboNZBCreatorPercentage), 0);
    gtk_combo_box_set_active(GTK_COMBO_BOX(comboNZBCreatorFileType), 0);

    store = gtk_tree_store_new( LAST_FIELD,
                                G_TYPE_INT64    /* FIELD_COLLECTION_ID */,
                                G_TYPE_STRING   /* FIELD_COLLECTION_NAME */,
                                G_TYPE_STRING   /* FIELD_SHORT_COLLECTION_NAME */,
                                G_TYPE_STRING   /* FIELD_SUBJECT */,
                                G_TYPE_STRING   /* FIELD_POSTER */,
                                G_TYPE_STRING   /* FIELD_NEWSGROUP */,
                                G_TYPE_INT64    /* FIELD_FILE_ID */,
                                G_TYPE_STRING   /* FIELD_FILESIZE_STR */,
                                G_TYPE_INT      /* FIELD_STAMP */,
                                G_TYPE_STRING   /* FIELD_STAMP_STR */,
                                G_TYPE_INT      /* FIELD_COMPLETE_PERCENTAGE */,
                                G_TYPE_BOOLEAN  /* FIELD_SELECTED */);

    cellRenderer = gtk_cell_renderer_toggle_new();
    columns[FIELD_SELECTED] = gtk_tree_view_column_new_with_attributes("", cellRenderer, "active", FIELD_SELECTED, NULL);
    gtk_tree_view_column_set_resizable(columns[FIELD_SELECTED], TRUE);
    gtk_tree_view_column_set_min_width(columns[FIELD_SELECTED], 30);
    gtk_tree_view_column_set_reorderable(columns[FIELD_SELECTED], TRUE);
    gtk_tree_view_insert_column(GTK_TREE_VIEW(treeNZBCreatorResults), columns[FIELD_SELECTED], -1);
    g_signal_connect(GTK_CELL_RENDERER_TOGGLE(cellRenderer), "toggled", G_CALLBACK(on_column_select_toggled), store);

    cellRenderer = gtk_cell_renderer_progress_new();
    columns[FIELD_COMPLETE_PERCENTAGE] = gtk_tree_view_column_new_with_attributes(_("% complete"), cellRenderer, "value", FIELD_COMPLETE_PERCENTAGE, NULL);
    gtk_tree_view_column_set_resizable(columns[FIELD_COMPLETE_PERCENTAGE], TRUE);
    gtk_tree_view_column_set_reorderable(columns[FIELD_COMPLETE_PERCENTAGE], TRUE);
    gtk_tree_view_insert_column(GTK_TREE_VIEW(treeNZBCreatorResults), columns[FIELD_COMPLETE_PERCENTAGE], -1);

    cellRenderer = gtk_cell_renderer_text_new();
    columns[FIELD_SUBJECT] = gtk_tree_view_column_new_with_attributes(_("Subject"), cellRenderer, "text", FIELD_SUBJECT, NULL);
    gtk_tree_view_column_set_resizable(columns[FIELD_SUBJECT], TRUE);
    gtk_tree_view_column_set_reorderable(columns[FIELD_SUBJECT], TRUE);
    gtk_tree_view_insert_column(GTK_TREE_VIEW(treeNZBCreatorResults), columns[FIELD_SUBJECT], -1);

    cellRenderer = gtk_cell_renderer_text_new();
    columns[FIELD_POSTER] = gtk_tree_view_column_new_with_attributes(_("Poster"), cellRenderer, "text", FIELD_POSTER, NULL);
    gtk_tree_view_column_set_resizable(columns[FIELD_POSTER], TRUE);
    gtk_tree_view_column_set_reorderable(columns[FIELD_POSTER], TRUE);
    gtk_tree_view_insert_column(GTK_TREE_VIEW(treeNZBCreatorResults), columns[FIELD_POSTER], -1);

    cellRenderer = gtk_cell_renderer_text_new();
    columns[FIELD_NEWSGROUP] = gtk_tree_view_column_new_with_attributes(_("Newsgroup"), cellRenderer, "text", FIELD_NEWSGROUP, NULL);
    gtk_tree_view_column_set_resizable(columns[FIELD_NEWSGROUP], TRUE);
    gtk_tree_view_column_set_reorderable(columns[FIELD_NEWSGROUP], TRUE);
    gtk_tree_view_insert_column(GTK_TREE_VIEW(treeNZBCreatorResults), columns[FIELD_NEWSGROUP], -1);

    cellRenderer = gtk_cell_renderer_text_new();
    columns[FIELD_FILESIZE_STR] = gtk_tree_view_column_new_with_attributes(_("File size"), cellRenderer, "text", FIELD_FILESIZE_STR, NULL);
    gtk_tree_view_column_set_resizable(columns[FIELD_FILESIZE_STR], TRUE);
    gtk_tree_view_column_set_reorderable(columns[FIELD_FILESIZE_STR], TRUE);
    gtk_tree_view_insert_column(GTK_TREE_VIEW(treeNZBCreatorResults), columns[FIELD_FILESIZE_STR], -1);

    cellRenderer = gtk_cell_renderer_text_new();
    columns[FIELD_STAMP_STR] = gtk_tree_view_column_new_with_attributes(_("Stamp"), cellRenderer, "text", FIELD_STAMP_STR, NULL);
    gtk_tree_view_column_set_resizable(columns[FIELD_STAMP_STR], TRUE);
    gtk_tree_view_column_set_reorderable(columns[FIELD_STAMP_STR], TRUE);
    gtk_tree_view_insert_column(GTK_TREE_VIEW(treeNZBCreatorResults), columns[FIELD_STAMP_STR], -1);

    gtk_tree_view_set_model(GTK_TREE_VIEW(treeNZBCreatorResults), GTK_TREE_MODEL(store));

    /* Retrieve the list of available newsgroups and maintenance information in a seperate thread */
    g_return_if_fail(g_thread_create(fetch_newsgroups_func, NULL, FALSE, NULL));

    g_signal_connect(treeNZBCreatorResults, "row-expanded", G_CALLBACK(on_row_expanded), NULL);

    pool_details = g_thread_pool_new(get_collection_details_thread, NULL, 5, FALSE, NULL);

    results_per_page = 0;
    memset(&cached_search_opts, 0, sizeof(cached_search_opts));
#else
    GtkWidget *vboxNZBCreatorTab = nntpgrab_gui_base_get_widget("vboxNZBCreatorTab");
    gtk_widget_hide(vboxNZBCreatorTab);
#endif
}
