#ifndef WIDGETDOWNLOADQUEUE_H
#define WIDGETDOWNLOADQUEUE_H

#include <QWidget>
#include <QNNTPGrabGlue.h>
#include <QTreeWidgetItem>
#include "ProgressBarDelegateDownloadQueue.h"
#include "DownloadQueueModel.h"

namespace Ui {
    class WidgetDownloadQueue;
}

class WidgetDownloadQueue : public QWidget {
    Q_OBJECT
public:
    WidgetDownloadQueue(QNNTPGrabGlue *glue, QAction *actionRemoveSelectedFiles, QWidget *parent = 0);
    ~WidgetDownloadQueue();

    void hideField(DownloadQueueItem::field f);
    void showField(DownloadQueueItem::field f);

protected:
    void changeEvent(QEvent *e);
    bool eventFilter(QObject *watched, QEvent *e);

private:
    Ui::WidgetDownloadQueue *ui;
    QNNTPGrabGlue *glue;
    ProgressBarDelegateDownloadQueue *progressbarDelegate;
    QAction *actionRemoveSelectedFiles;
    DownloadQueueModel *model;

    enum moveDirection {
        MOVE_DIRECTION_TOP,
        MOVE_DIRECTION_UP,
        MOVE_DIRECTION_DOWN,
        MOVE_DIRECTION_BOTTOM
    };

    void doMoveItem(QString collection_name, QString subject, int new_position);
    int calculateNewPosition(int current_position, moveDirection direction);
    void processSelectionAndMoveItems(moveDirection direction);

#if 0
    QTreeWidgetItem *getTreeWidgetByCollection(QString collection_name);
    QTreeWidgetItem *getTreeWidgetBySubject(QTreeWidgetItem *collection_node, QString subject);
#endif

private slots:
    void onRequestColumnResize(DownloadQueueItem::field column);

    void btnQueueMoveToTop_clicked(void);
    void btnQueueMoveUp_clicked(void);
    void btnQueueMoveDown_clicked(void);
    void btnQueueMoveToBottom_clicked(void);

    void onSelectionChanged(const QItemSelection &new_selection, const QItemSelection &oldSelection);

    void actionRemoveSelectedFiles_activated(void);
    void actionRestartSelectedItems_activated(void);
    void actionMarkItemOptional_activated(void);
    void actionOpenDownloadFolder_activated(void);
    void actionExecuteSelectedItem_activated(void);

    void showContextMenu(const QPoint &position);
};

#endif // WIDGETDOWNLOADQUEUE_H
